;
; cpmio.s - CP/M file I/O routines
;
        .module cpmio

bdos =		0x0005

b_reset =   0x00
b_conio =   0x06
b_printf =	0x09
b_setdma =	0x1a
b_resdsc =	0x0d
b_open =	0x0f
b_create =	0x16
b_read =	0x14
b_write =	0x15
b_close =	0x10
b_delete =	0x13
b_rename =	0x17
b_sff =		0x11
b_sfn =		0x12
b_cfs = 	0x23
b_logdrv =  0x18    
b_curdrv =  0x19    
b_seldrv =  0x0E
b_drvpar =  0x1F
b_user =    0x20
    
        .area   _CODE

; calling BDOS with parameter DE is such a common thing
bdos_de:
	pop	hl
	pop	de
	push	de
	push	hl
	call	#bdos
	ld	l,a
	ret

; void bdos_exit(void)
_bdos_exit::
    ld  c,#b_reset
    jp  bdos

; unsigned char bdos_inkey(void)
_bdos_inkey::
    ld  c,#b_conio
    ld  e,#0xff
    jp  bdos
    
; void bdos_printf(const char *str)
_bdos_printf::
	ld	c,#b_printf
	jp	bdos_de

; void bdos_set_dma_address(void *addr)
_bdos_set_dma_address::
	ld	c,#b_setdma
	jp	bdos_de

; void bdos_reset_disc_drives(void)
_bdos_reset_disc_drives::
	ld	c,#b_resdsc
	jp	bdos

; unsigned char bdos_open_file(FCB *fcb)
_bdos_open_file::
	ld	c,#b_open
	jp	bdos_de

; unsigned char bdos_create_file(FCB *fcb)
_bdos_create_file::
	ld	c,#b_create
	jp	bdos_de

; unsigned char bdos_read_file(FCB *fcb)
_bdos_read_file::
	ld	c,#b_read
	jp	bdos_de

; unsigned char bdos_write_file(FCB *fcb)
_bdos_write_file::
	ld	c,#b_write
	jp	bdos_de

; unsigned char bdos_close_file(FCB *fcb)
_bdos_close_file::
	ld	c,#b_close
	jp	bdos_de

; unsigned char bdos_delete_files(FCB *fcb)
_bdos_delete_files::
	ld	c,#b_delete
	jp	bdos_de

; unsigned char bdos_rename_file(FCB *fcb)
_bdos_rename_file::
	ld	c,#b_rename
	jp	bdos_de

; unsigned char bdos_search_for_first(FCB *fcb)
_bdos_search_for_first::
	ld	c,#b_sff
	jp	bdos_de

; unsigned char bdos_search_for_next(FCB *fcb)
_bdos_search_for_next::
	ld	c,#b_sfn
	jp	bdos_de

; unsigned char bdos_file_size(FCB *fcb)
_bdos_file_size::
	ld	c,#b_cfs
	jp	bdos_de

; unsigned char bdos_current_drive (void)
_bdos_current_drive::
	ld	c,#b_curdrv
	call	#bdos
	ld	l,a
	ret

; unsigned int bdos_logged_drives (void)
_bdos_logged_drives::
	ld	c,#b_logdrv
	call	#bdos
	ret

; void bdos_select_drive (unsigned char drv)
_bdos_select_drive::
    pop hl
    pop de
    push de
    push hl
    ld	c,#b_seldrv
	call	#bdos
    ret

; DPB * bdos_disk_parameters (void)
_bdos_disk_parameters::
	ld	c,#b_drvpar
	call	#bdos
	ret

; unsigned char bdos_get_user (void)
_bdos_get_user::
	ld	c,#b_user
    ld  e,#0xff
	call	#bdos
	ld	l,a
	ret

; extern unsigned int get_sp (void)
_get_sp::
    ld  hl,#0
    add hl,sp
    ret

    
; void memu_char (char ch)
_memu_char::
    pop hl
    pop de
    push de
    push hl
    ld a,e
    out (0xEE), a
    ret

; void memu_hex (unsigned char ch)
_memu_hex::
    pop hl
    pop de
    push de
    push hl
    ld a,e
    out (0xEF), a
    ret
