/*

httpd.c - Four channel web server - Initialise and start services

*/

/*...sincludes:0:*/
#include <stdio.h>
#include "types.h"
#include "interrupts.h"
#include "fiber.h"
#include "cpmio.h"
#include "net.h"
#include "http.h"
#include "log.h"

/*...vtypes\46\h:0:*/
/*...vinterrupts\46\h:0:*/
/*...vfiber\46\h:0:*/
/*...vclock\46\h:0:*/
/*...vcpmio\46\h:0:*/
/*...vnet\46\h:0:*/
/*...vecho\46\h:0:*/
/*...vchargen\46\h:0:*/
/*...vhttp\46\h:0:*/
/*...e*/

static HTTP http[4];

static void first(void *nfp)
	{
    int i;
	nfp=nfp;
    for (i = 0; i < 4; ++i)
        {
        http[i].allow_post   = TRUE;
        http[i].allow_delete = TRUE;
        http_start(&http[i]);
        }
	}

static FIBER first_fiber = { NULL, first, NULL };

void quit(void)
    {
    log_close();
    bdos_exit();
    }

void parse(void)
    {
    const char *ctail = (char *) 0x80;
    const char *cend = ctail + ctail[0] + 1;
    const char *ps = ctail + 1;
    while ( ps < cend )
        {
        if ( *ps == ' ' )
            {
            ++ps;
            }
        else if ( *ps == '-' )
            {
            ++ps;
            switch (*ps)
                {
                case 'L':
                case 'l':
                    ++ps;
                    ps = log_open (ps, cend);
                    break;
                case 'H':
                case 'h':
                    ++ps;
                    ps = set_home (ps, cend);
                    break;
                default:
                    log_fatal ("Invalid parameter\r\n$");
                }
            }
        else
            {
            ps = log_open (ps, cend);
            }
        }
    }

int main(void)
	{
	bdos_printf("Hypertext Transfer Protocol Daemon" VERSION "\r\n$");
    parse();
	fiber_init();
	interrupts_enable();
	net_init();
	http_init();
	fiber_enqueue(&first_fiber);
	for ( ;; )
		{
        if ( bdos_inkey() == 'q' ) quit();
		net_handle_interrupts();
		fiber_drain();
		}
	}
