/* log.c - Routines for logging messages to a file */

#include "cpmio.h"
#include "log.h"

#define LOG_LEN     128

static FCB log_fcb;
static char log_buf[LOG_LEN+1];
static word log_pos = 0;
static char *log_ptr = log_buf;
static byte log_fil = 0;
int log_nshow = 0;

void log_fatal(const char *err)
    {
    bdos_printf(err);
    bdos_exit();
    }

const char * log_open(const char *ps, const char *cend)
    {
    char *pd = log_fcb.filename;
    byte n = 8;
    log_fcb.drive = 0;
    log_fcb.filename[0] = ' ';
    log_fcb.filename[1] = ' ';
    log_fcb.filename[2] = ' ';
    log_fcb.filename[3] = ' ';
    log_fcb.filename[4] = ' ';
    log_fcb.filename[5] = ' ';
    log_fcb.filename[6] = ' ';
    log_fcb.filename[7] = ' ';
    log_fcb.type[0] = ' ';
    log_fcb.type[1] = ' ';
    log_fcb.type[2] = ' ';
    log_fcb.ex = 0;
    log_fcb.s1 = 0;
    log_fcb.s2 = 0;
    log_fcb.cr = 0;
    while ((ps < cend) && (*ps == ' ')) ++ps;
    while ((ps < cend) && (n > 0))
        {
        char c = *(ps++);
        if ( c == '.' )
            {
            pd = log_fcb.type;
            n = 3;
            }
        else if ( c <= ' ' )
            {
            break;
            }
        else
            {
            if ((c >= 'a') && (c <= 'z')) c -= 0x20;
            *(pd++) = c;
            --n;
            }
        }
    if ((ps != cend) && (*ps > ' ')) log_fatal("Invalid log file name\r\n$");
    bdos_delete_files(&log_fcb);
    log_fcb.ex = 0;
    log_fcb.s1 = 0;
    log_fcb.s2 = 0;
    log_fcb.cr = 0;
    if ( bdos_create_file(&log_fcb) == 255 ) log_fatal("Unable to create log file\r\n$");
    log_fil = 1;
    log_pos = 0;
    log_ptr = log_buf;
    return ps;
    }

static void log_show(void)
    {
    if (log_pos > 0)
        {
        char *ps = log_ptr;
        ++log_nshow;
        log_buf[log_pos] = '$';
        bdos_printf(log_ptr);
#ifdef MEMU_LOG
        while (*ps != '$')
            {
            if ( *ps != '\r' ) memu_char (*ps);
            ++ps;
            }
#endif
        }
    }

static void log_flush(void)
    {
    log_show();
    if ( log_fil )
        {
        bdos_set_dma_address(log_buf);
        if ( bdos_write_file(&log_fcb) != 0 ) log_fatal("Error writing log file\r\n$");
        }
    log_pos = 0;
    log_ptr = log_buf;
    }

static void log_chr(char c)
    {
    log_buf[log_pos] = c;
    if ( ++log_pos == LOG_LEN ) log_flush();
    }

void log_printf(const char *ps)
    {
    log_ptr = &log_buf[log_pos];
    while ( ( *ps ) && ( *ps != '$' ) )
        {
        log_buf[log_pos] = *(ps++);
        if ( ++log_pos == LOG_LEN ) log_flush();
        }
    log_show();
    }

void log_line(const char *ps)
    {
    log_ptr = &log_buf[log_pos];
    while (*ps)
        {
        log_buf[log_pos] = *ps;
        if ( ++log_pos == LOG_LEN ) log_flush();
        if ( *ps == '\n' ) break;
        ++ps;
        }
    log_show();
    }

void log_str(const char *ps)
    {
    log_ptr = &log_buf[log_pos];
    while ( *ps )
        {
        log_buf[log_pos] = *(ps++);
        if ( ++log_pos == LOG_LEN ) log_flush();
        }
    log_show();
    }

void log_strn(const char *ps, word n)
    {
    log_ptr = &log_buf[log_pos];
    while ( ( *ps ) && ( n > 0 ) )
        {
        log_buf[log_pos] = *(ps++);
        if ( ++log_pos == LOG_LEN ) log_flush();
        --n;
        }
    log_show();
    }

static void log_hex(byte b)
    {
    byte h = b >> 4;
    log_chr((h < 10) ? h + '0' : h - 10 + 'A');
    h = b & 0x0F;
    log_chr((h < 10) ? h + '0' : h - 10 + 'A');
    }

void log_bin(const char *ps, word n)
    {
    log_ptr = &log_buf[log_pos];
    do
        {
        char c = *(ps++);
        if (( c >= ' ' ) && ( c < 0x7F ))
            {
            log_chr (c);
            }
        else
            {
            log_chr ('\\');
            log_hex (c);
            }
        }
    while ( --n );
    log_show();
    }

void log_byte(byte b)
    {
    log_ptr = &log_buf[log_pos];
    log_hex(b);
    log_show();
    }

void log_word(word w)
    {
    log_ptr = &log_buf[log_pos];
    log_hex(w >> 8);
    log_hex(w & 0xFF);
    log_show();
    }

static void log_udgt(word *w, byte *f, word d)
    {
    char c = '0';
    while ( *w >= d )
        {
        *w -= d;
        ++c;
        }
    if (( c > '0' ) || ( *f ))
        {
        log_chr(c);
        *f = 1;
        }
    }

static void log_udec(word w)
    {
    byte f = 0;
    log_udgt(&w, &f, 10000);
    log_udgt(&w, &f, 1000);
    log_udgt(&w, &f, 100);
    log_udgt(&w, &f, 10);
    log_chr(w + '0');
    }

void log_uint(word w)
    {
    log_ptr = &log_buf[log_pos];
    log_udec(w);
    log_show();
    }

void log_int(int i)
    {
    log_ptr = &log_buf[log_pos];
    if ( i < 0 )
        {
        log_chr('-');
        i = -i;
        }
    log_udec(i);
    log_show();
    }

void log_close(void)
    {
    if ( log_fil )
        {
        if ( log_pos > 0 )
            {
            log_buf[log_pos] = 0x1A;
            log_pos = 0;
            log_flush();
            }
        bdos_close_file(&log_fcb);
        }
    }
