;;  wiz.c - Access to WIZnet on NFX
;;   
;;  SDCC can generate inline IN and OUT instructions for fixed port addresses.
;;  However there is no apparent way of doing so for dynamically assigned addresses.
;;  One option would have been to write subroutines to perform the port read and write
;;  but that would introduce subroutine call overhead and slow down the code.
;;   
;;  Instead, the ASM code generated by compiling the fixed port code has been
;;  copied and hand editted to replace IN A,(#) and OUT (#), A by IN A,(C) and OUT (C),A.
;;   
;;  The ASM code has also been optimised in other ways, including inlining wiz_set_addr()
;;  and the contents of "wizio.s".
;;
	.module wiz
	.optsdcc -mz80
	
	.globl _wiz_config
	.globl _wiz_reset
	.globl _wiz_set_addr
	.globl _wiz_write
	.globl _wiz_write_word
	.globl _wiz_write_n
	.globl _wiz_read
	.globl _wiz_read_word
	.globl _wiz_read_n

	.area _DATA
_wiz_port_control:
	.ds 1
_wiz_port_addr_msb:
	.ds 1
_wiz_port_addr_lsb:
	.ds 1
_wiz_port_data:
	.ds 1

	.area _CODE
;wiz.c:86: void wiz_config (byte port)
; ---------------------------------
; Function wiz_config
; ---------------------------------
_wiz_config::
;wiz.c:88: byte pt = port;
	ld	iy, #2
	add	iy, sp
	ld	a, 0 (iy)
;wiz.c:89: wiz_port_control = pt;
	ld	(_wiz_port_control+0), a
;wiz.c:90: wiz_port_addr_msb = ++pt;
	inc	a
	ld	(_wiz_port_addr_msb+0), a
;wiz.c:91: wiz_port_addr_lsb = ++pt;
	inc	a
	ld	(_wiz_port_addr_lsb+0), a
;wiz.c:92: wiz_port_data = ++pt;
	inc	a
	ld	(_wiz_port_data+0), a
;wiz.c:93: }
	ret

;wiz.c:97: void wiz_reset(void)
; ---------------------------------
; Function wiz_reset
; ---------------------------------
_wiz_reset::
;wiz.c:80:	NFX_WIZ_CONTROL = 0x80 | 0x01 | 0x02;
	ld	a, #0x83
;	out (_NFX_WIZ_CONTROL), a
	ld	bc, (_wiz_port_control)
	out	(c), a
;wiz.c:82:	while ( NFX_WIZ_CONTROL & 0x80 )
00101$:
;	in a, (_NFX_WIZ_CONTROL)
	in	a, (c)
	rlca
	jr	C,00101$
;wiz.c:91:	NFX_WIZ_CONTROL = 0x01 | 0x02;
	ld	a, #0x03
;	out (_NFX_WIZ_CONTROL), a
	out	(c), a
;wiz.c:92:	}
	ret

;wiz.c:136: void wiz_set_addr(word addr)
; ---------------------------------
; Function wiz_set_addr
; ---------------------------------
_wiz_set_addr::
;wiz.c:97:	NFX_WIZ_ADDR_MSB = (byte) (addr>>8);
	ld	iy, #2
	add	iy, sp
	ld	a, 1 (iy)
;	out (_NFX_WIZ_ADDR_MSB), a
	ld	bc, (_wiz_port_addr_msb)
	out	(c), a
;wiz.c:98:	NFX_WIZ_ADDR_LSB = (byte) addr ;
	ld	a, 0 (iy)
;	out (_NFX_WIZ_ADDR_LSB), a
	inc	c
	out	(c), a
;wiz.c:99:	}

;wiz.c:163: void wiz_write(word addr, byte value) __naked
; ---------------------------------
; Function wiz_write
; ---------------------------------
_wiz_write::
;wiz.c:169:	NFX_WIZ_ADDR_MSB = (byte) (addr>>8);
	ld	iy, #2
	add	iy, sp
	ld	a, 1 (iy)
;	out (_NFX_WIZ_ADDR_MSB), a
	ld	bc, (_wiz_port_addr_msb)
	out	(c), a
;wiz.c:170:	NFX_WIZ_ADDR_LSB = (byte) addr ;
	ld	a, 0 (iy)
;	out (_NFX_WIZ_ADDR_LSB), a
	inc	c
	out	(c), a
;wiz.c:171:	NFX_WIZ_DATA = value;
	ld	a, 2 (iy)
;	out (_NFX_WIZ_DATA), a
	inc	c
	out	(c), a
;wiz.c:173:	}
	ret
;wiz.c:196: }

;wiz.c:199: void wiz_write_word(word addr, word ptr)
; ---------------------------------
; Function wiz_write_word
; ---------------------------------
_wiz_write_word::
;wiz.c:183:	NFX_WIZ_ADDR_MSB = (byte) (addr>>8);
	ld	iy, #2
	add	iy, sp
	ld	a, 1 (iy)
;	out (_NFX_WIZ_ADDR_MSB), a
	ld	bc, (_wiz_port_addr_msb)
	out	(c), a
;wiz.c:184:	NFX_WIZ_ADDR_LSB = (byte) addr ;
	ld	a, 0 (iy)
;	out (_NFX_WIZ_ADDR_LSB), a
	inc	c
	out	(c), a
;wiz.c:185:	NFX_WIZ_DATA = (byte) (ptr>>8);
	ld	a, 3 (iy)
;	out (_NFX_WIZ_DATA), a
	inc	c
	out	(c), a
;wiz.c:186:	NFX_WIZ_DATA = (byte) ptr ;
	ld	a, 2 (iy)
;	out (_NFX_WIZ_DATA), a
	out	(c), a
;wiz.c:188:	}
	ret

;wiz.c:240: void wiz_write_n(word addr, const byte *values, word n)
; ---------------------------------
; Function wiz_write_n
; ---------------------------------
_wiz_write_n::
;wiz.c:183:	NFX_WIZ_ADDR_MSB = (byte) (addr>>8);
	ld	iy, #2
	add	iy, sp
	ld	a, 1 (iy)
;	out (_NFX_WIZ_ADDR_MSB), a
	ld	bc, (_wiz_port_addr_msb)
	out	(c), a
;wiz.c:184:	NFX_WIZ_ADDR_LSB = (byte) addr ;
	ld	a, 0 (iy)
;	out (_NFX_WIZ_ADDR_LSB), a
	inc	c
	out	(c), a
	inc	c
;wiz.c:195:	wizio_out(values, n, NFX_WIZ_DATA_PORT);
	ld	l,2(iy)
	ld	h,3(iy)
	ld	e,4(iy)
	ld	d,5(iy)
	inc	d
	dec	d
	jr	z,00002$
	ld	b,#0
00001$:
	otir
	dec	d
	jr	nz,00001$
00002$:
	inc	e
	dec	e
	ret	z
	ld	b,e
	otir
;wiz.c:291: }
	ret

;wiz.c:294: byte wiz_read(word addr)
; ---------------------------------
; Function wiz_read
; ---------------------------------
_wiz_read::
;wiz.c:210:	NFX_WIZ_ADDR_MSB = (byte) (addr>>8);
	ld	iy, #2
	add	iy, sp
	ld	a, 1 (iy)
;	out (_NFX_WIZ_ADDR_MSB), a
	ld	bc, (_wiz_port_addr_msb)
	out	(c), a
;wiz.c:211:	NFX_WIZ_ADDR_LSB = (byte) addr ;
	ld	a, 0 (iy)
;	out (_NFX_WIZ_ADDR_LSB), a
	inc	c
	out	(c), a
;wiz.c:212:	return NFX_WIZ_DATA;
;	in a, (_NFX_WIZ_DATA)
;	ld l, a
	inc	c
	in	l, (c)
;wiz.c:214:	}
	ret

;wiz.c:329: word wiz_read_word(word addr)
; ---------------------------------
; Function wiz_read_word
; ---------------------------------
_wiz_read_word::
;wiz.c:210:	NFX_WIZ_ADDR_MSB = (byte) (addr>>8);
	ld	iy, #2
	add	iy, sp
	ld	a, 1 (iy)
;	out (_NFX_WIZ_ADDR_MSB), a
	ld	bc, (_wiz_port_addr_msb)
	out	(c), a
;wiz.c:211:	NFX_WIZ_ADDR_LSB = (byte) addr ;
	ld	a, 0 (iy)
;	out (_NFX_WIZ_ADDR_LSB), a
	inc	c
	out	(c), a
;wiz.c:227:	r0 = NFX_WIZ_DATA;
;wiz.c:228:	r1 = NFX_WIZ_DATA;
;wiz.c:229:	return ((word)r0)<<8 | (word)r1;
	inc	c
	in	h, (c)
	in	l, (c)
;wiz.c:231:	}
	ret

;wiz.c:368: void wiz_read_n(word addr, byte *values, word n)
; ---------------------------------
; Function wiz_read_n
; ---------------------------------
_wiz_read_n::
;wiz.c:210:	NFX_WIZ_ADDR_MSB = (byte) (addr>>8);
	ld	iy, #2
	add	iy, sp
	ld	a, 1 (iy)
;	out (_NFX_WIZ_ADDR_MSB), a
	ld	bc, (_wiz_port_addr_msb)
	out	(c), a
;wiz.c:211:	NFX_WIZ_ADDR_LSB = (byte) addr ;
	ld	a, 0 (iy)
;	out (_NFX_WIZ_ADDR_LSB), a
	inc	c
	out	(c), a
	inc	c
;wiz.c:238:	wizio_in(values, n, NFX_WIZ_DATA_PORT);
	ld	l,2(iy)
	ld	h,3(iy)
	ld	e,4(iy)
	ld	d,5(iy)
	inc	d
	dec	d
	jr	z,00002$
	ld	b,#0
00001$:
	inir
	dec	d
	jr	nz,00001$
00002$:
	inc	e
	dec	e
	ret	z
	ld	b,e
	inir
	ret
